@@ -42,6 +42,7 @@ class AddDialogTest : public QObject
     Q_OBJECT
 
 private Q_SLOTS:
+    void initTestCase();
     void testBasicWidgets_data();
     void testBasicWidgets();
 
@@ -49,6 +50,12 @@ private:
     PluginManager m_pluginManager;
 };
 
+void AddDialogTest::initTestCase()
+{
+    // Avoid a runtime dependency on KLauncher.
+    qputenv("KDE_FORK_SLAVES", "yes");
+}
+
 void AddDialogTest::testBasicWidgets_data()
 {
     QTest::addColumn<QString>("mimeType");
@@ -137,12 +144,12 @@ void AddDialogTest::testBasicWidgets()
 
     if (supportsCompLevel) {
         // Test that the value set by slider is exported from AddDialog.
-        QCOMPARE(dialog->compressionOptions()[QStringLiteral("CompressionLevel")].toInt(), changeToCompLevel);
+        QCOMPARE(dialog->compressionOptions().compressionLevel(), changeToCompLevel);
     }
 
     // Test that passing a compression level in ctor works.
     CompressionOptions opts;
-    opts[QStringLiteral("CompressionLevel")] = initialCompLevel;
+    opts.setCompressionLevel(initialCompLevel);
 
     dialog = new AddDialog(Q_NULLPTR, QString(), QUrl(), mime, opts);
     dialog->slotOpenOptions();
