@@ -46,6 +46,8 @@ namespace Kerfuffle
 
 class ExtractionDialogUI: public QFrame, public Ui::ExtractionDialog
 {
+    Q_OBJECT
+
 public:
     ExtractionDialogUI(QWidget *parent = 0)
             : QFrame(parent) {
@@ -201,6 +203,21 @@ QString ExtractionDialog::subfolder() const
     return m_ui->subfolder->text();
 }
 
+void ExtractionDialog::setBusyGui()
+{
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+    fileWidget->setEnabled(false);
+    m_ui->setEnabled(false);
+    // TODO: tell the user why the dialog is busy (e.g. "archive is being loaded").
+}
+
+void ExtractionDialog::setReadyGui()
+{
+    QApplication::restoreOverrideCursor();
+    fileWidget->setEnabled(true);
+    m_ui->setEnabled(true);
+}
+
 ExtractionDialog::~ExtractionDialog()
 {
     delete m_ui;
@@ -274,7 +291,11 @@ QUrl ExtractionDialog::destinationDirectory() const
 {
     if (extractToSubfolder()) {
         QUrl subUrl = fileWidget->baseUrl();
-        subUrl.setPath(subUrl.path() + QLatin1Char('/') + subfolder());
+        if (subUrl.path().endsWith(QDir::separator())) {
+            subUrl.setPath(subUrl.path() + subfolder());
+        } else {
+            subUrl.setPath(subUrl.path() + QDir::separator() + subfolder());
+        }
 
         return subUrl;
     } else {
@@ -305,4 +326,7 @@ void ExtractionDialog::restoreWindowSize()
   KConfigGroup group(KSharedConfig::openConfig(), "ExtractDialog");
   KWindowConfig::restoreWindowSize(windowHandle(), group);
 }
+
 }
+
+#include "extractiondialog.moc"
