@@ -1,7 +1,7 @@
 /*
  * ark -- archiver for the KDE project
  *
- * Copyright (C) 2016 Elvis Angelaccio <elvis.angelaccio@kdemail.net>
+ * Copyright (C) 2016 Elvis Angelaccio <elvis.angelaccio@kde.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -170,13 +170,23 @@ QVector<Plugin*> PluginManager::filterBy(const QVector<Plugin*> &plugins, const
 void PluginManager::loadPlugins()
 {
     const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kerfuffle"));
+    // This class might be used from executables other than ark (e.g. the tests),
+    // so we need to specify the name of the config file.
     // TODO: once we have a GUI in the settings dialog,
     // use this group to write whether a plugin gets disabled.
-    const KConfigGroup conf(KSharedConfig::openConfig(), "EnabledPlugins");
+    const KConfigGroup conf(KSharedConfig::openConfig(QStringLiteral("arkrc")), "EnabledPlugins");
 
+    QSet<QString> addedPlugins;
     foreach (const KPluginMetaData &metaData, plugins) {
+        const auto pluginId = metaData.pluginId();
+        // Filter out duplicate plugins.
+        if (addedPlugins.contains(pluginId)) {
+            continue;
+        }
+
         Plugin *plugin = new Plugin(this, metaData);
-        plugin->setEnabled(conf.readEntry(metaData.pluginId(), true));
+        plugin->setEnabled(conf.readEntry(pluginId, true));
+        addedPlugins << pluginId;
         m_plugins << plugin;
     }
 }
