@@ -42,6 +42,8 @@ namespace Kerfuffle
 {
 class PropertiesDialogUI: public QWidget, public Ui::PropertiesDialog
 {
+    Q_OBJECT
+
 public:
     PropertiesDialogUI(QWidget *parent = 0)
             : QWidget(parent) {
@@ -63,6 +65,7 @@ PropertiesDialog::PropertiesDialog(QWidget *parent, Archive *archive, qulonglong
     m_ui->lblArchiveName->setText(archive->fileName());
     m_ui->lblArchiveType->setText(archive->mimeType().comment());
     m_ui->lblMimetype->setText(archive->mimeType().name());
+    m_ui->lblCompressionMethods->setText(archive->property("compressionMethods").toStringList().join(QStringLiteral(", ")));
     m_ui->lblReadOnly->setText(archive->isReadOnly() ?  i18n("yes") : i18n("no"));
     m_ui->lblMultiVolume->setText(archive->isMultiVolume() ? i18n("yes (%1 volumes)", archive->numberOfVolumes()) : i18n("no"));
     m_ui->lblHasComment->setText(archive->hasComment() ?  i18n("yes") : i18n("no"));
@@ -81,10 +84,10 @@ PropertiesDialog::PropertiesDialog(QWidget *parent, Archive *archive, qulonglong
         m_ui->lblPasswordProtected->setText(i18n("no"));
         break;
     case Archive::Encrypted:
-        m_ui->lblPasswordProtected->setText(i18n("yes (excluding the list of files)"));
+        m_ui->lblPasswordProtected->setText(i18n("only file contents (%1)", archive->property("encryptionMethods").toStringList().join(QStringLiteral(", "))));
         break;
     case Archive::HeaderEncrypted:
-        m_ui->lblPasswordProtected->setText(i18n("yes (including the list of files)"));
+        m_ui->lblPasswordProtected->setText(i18n("yes (%1)", archive->property("encryptionMethods").toStringList().join(QStringLiteral(", "))));
         break;
     }
 
@@ -138,4 +141,5 @@ void PropertiesDialog::showChecksum(QCryptographicHash::Algorithm algorithm, con
     futureWatcher->setFuture(future);
 }
 
+#include "propertiesdialog.moc"
 }
