@@ -27,6 +27,7 @@
 #define SINGLEFILEPLUGIN_H
 
 #include "kerfuffle/archiveinterface.h"
+#include "kerfuffle/archiveentry.h"
 
 class LibSingleFileInterface : public Kerfuffle::ReadOnlyArchiveInterface
 {
@@ -38,7 +39,7 @@ public:
 
     virtual bool list() Q_DECL_OVERRIDE;
     virtual bool testArchive() Q_DECL_OVERRIDE;
-    virtual bool copyFiles(const QList<QVariant>& files, const QString& destinationDirectory, const Kerfuffle::ExtractionOptions& options) Q_DECL_OVERRIDE;
+    virtual bool extractFiles(const QVector<Kerfuffle::Archive::Entry*> &files, const QString &destinationDirectory, const Kerfuffle::ExtractionOptions &options) Q_DECL_OVERRIDE;
 
 protected:
     const QString uncompressedFileName() const;
